@@ -24,8 +24,9 @@ f_new_inst(void)
 }
 
 struct f_inst *
-f_new_dynamic_attr(int type, int f_type, int code)
+f_new_dynamic_attr(int type, int f_type UNUSED, int code)
 {
+  /* FIXME: Remove the f_type parameter? */
   struct f_inst *f = f_new_inst();
   f->aux = type;
   f->a2.i = code;
