@@ -50,7 +50,7 @@
 
 #define CMP_ERROR 999
 
-struct adata *
+static struct adata *
 adata_empty(struct linpool *pool)
 {
   struct adata *res = lp_alloc(pool, sizeof(struct adata));
@@ -81,6 +81,8 @@ pm_path_compare(struct f_path_mask *m1, struct f_path_mask *m2)
 int
 val_compare(struct f_val v1, struct f_val v2)
 {
+  int rc;
+
   if ((v1.type == T_VOID) && (v2.type == T_VOID))
     return 0;
   if (v1.type == T_VOID)	/* Hack for else */
@@ -100,8 +102,15 @@ val_compare(struct f_val v1, struct f_val v2)
     if (v1.val.i < v2.val.i) return -1;
     return 1;
   case T_IP:
-  case T_PREFIX:
     return ipa_compare(v1.val.px.ip, v2.val.px.ip);
+  case T_PREFIX:
+    if (rc = ipa_compare(v1.val.px.ip, v2.val.px.ip))
+      return rc;
+    if (v1.val.px.len < v2.val.px.len)
+      return -1;
+    if (v1.val.px.len > v2.val.px.len)
+      return 1;
+    return 0;
   case T_PATH_MASK:
     return pm_path_compare(v1.val.path_mask, v2.val.path_mask);
   default:
@@ -113,7 +122,7 @@ val_compare(struct f_val v1, struct f_val v2)
 /*
  * val_simple_in_range - check if @v1 ~ @v2 for everything except sets
  */ 
-int
+static int
 val_simple_in_range(struct f_val v1, struct f_val v2)
 {
   if ((v1.type == T_PATH) && (v2.type == T_PATH_MASK))
@@ -152,7 +161,7 @@ val_simple_in_range(struct f_val v1, struct f_val v2)
  * Checks if @v1 is element (|~| operator) of @v2. Sets are internally represented as balanced trees, see
  * |tree.c| module (this is not limited to sets, but for non-set cases, val_simple_in_range() is called early).
  */
-int
+static int
 val_in_range(struct f_val v1, struct f_val v2)
 {
   int res;
@@ -231,7 +240,7 @@ static rta *f_rta_copy;
 /*
  * rta_cow - prepare rta for modification by filter
  */
-void
+static void
 rta_cow(void)
 {
   if (!f_rta_copy) {
@@ -285,7 +294,7 @@ interpret(struct f_inst *what)
 {
   struct symbol *sym;
   struct f_val v1, v2, res;
-  int i,j,k;
+  int i;
 
   res.type = T_VOID;
   if (!what)
