@@ -1,7 +1,7 @@
 /*
  *	BIRD Library
  *
- *	(c) 1998--2000 Martin Mares <mj@ucw.cz>
+ *	(c) 1998--2004 Martin Mares <mj@ucw.cz>
  *
  *	Can be freely distributed and used under the terms of the GNU GPL.
  */
@@ -13,25 +13,28 @@
 
 #define OFFSETOF(s, i) ((unsigned int)&((s *)0)->i)
 #define SKIP_BACK(s, i, p) ((s *)((char *)p - OFFSETOF(s, i)))
-#define ALIGN(s, a) (((s)+a-1)&~(a-1))
+#define BIRD_ALIGN(s, a) (((s)+a-1)&~(a-1))
 
 /* Utility macros */
 
 #define MIN(a,b) (((a)<(b))?(a):(b))
 #define MAX(a,b) (((a)>(b))?(a):(b))
+#define ABS(a)   ((a)>=0 ? (a) : -(a))
 #define ARRAY_SIZE(a) (sizeof(a)/sizeof(*(a)))
 
 #ifndef NULL
 #define NULL ((void *) 0)
 #endif
 
-/* Functions which don't return */
+/* Macros for gcc attributes */
 
 #define NORET __attribute__((noreturn))
+#define UNUSED __attribute__((unused))
 
 /* Logging and dying */
 
-void log(char *msg, ...);
+#define log log_msg
+void log_msg(char *msg, ...);
 void die(char *msg, ...) NORET;
 void bug(char *msg, ...) NORET;
 
@@ -39,11 +42,11 @@ void bug(char *msg, ...) NORET;
 #define L_TRACE "\002"			/* Protocol tracing */
 #define L_INFO "\003"			/* Informational messages */
 #define L_REMOTE "\004"			/* Remote protocol errors */
-#define L_WARN "\004"			/* Local warnings */
-#define L_ERR "\005"			/* Local errors */
-#define L_AUTH "\006"			/* Authorization failed etc. */
-#define L_FATAL "\007"			/* Fatal errors */
-#define L_BUG "\010"			/* BIRD bugs */
+#define L_WARN "\005"			/* Local warnings */
+#define L_ERR "\006"			/* Local errors */
+#define L_AUTH "\007"			/* Authorization failed etc. */
+#define L_FATAL "\010"			/* Fatal errors */
+#define L_BUG "\011"			/* BIRD bugs */
 
 void debug(char *msg, ...);		/* Printf to debug output */
 
