@@ -90,7 +90,7 @@ linpool
 void *
 lp_alloc(linpool *m, unsigned size)
 {
-  byte *a = (byte *) ALIGN((unsigned long) m->ptr, CPU_STRUCT_ALIGN);
+  byte *a = (byte *) BIRD_ALIGN((unsigned long) m->ptr, CPU_STRUCT_ALIGN);
   byte *e = a + size;
 
   if (e <= m->end)
@@ -107,7 +107,7 @@ lp_alloc(linpool *m, unsigned size)
 	  c = xmalloc(sizeof(struct lp_chunk) + size);
 	  m->total_large += size;
 	  c->next = m->first_large;
-	  m->first_large = c->next;
+	  m->first_large = c;
 	  c->size = size;
 	}
       else
