@@ -106,6 +106,26 @@ pool_lookup(resource *P, unsigned long a)
   return NULL;
 }
 
+/**
+ * rmove - move a resource
+ * @res: resource
+ * @p: pool to move the resource to
+ *
+ * rmove() moves a resource from one pool to another.
+ */
+
+void rmove(void *res, pool *p)
+{
+  resource *r = res;
+
+  if (r)
+    {
+      if (r->n.next)
+        rem_node(&r->n);
+      add_tail(&p->inside, &r->n);
+    }
+}
+
 /**
  * rfree - free a resource
  * @res: resource
@@ -233,7 +253,7 @@ struct mblock {
   byte data[0];
 };
 
-static void mbl_free(resource *r)
+static void mbl_free(resource *r UNUSED)
 {
 }
 
