@@ -44,14 +44,13 @@
  * Routing tables always contain only cached &rta's.
  */
 
-#include <alloca.h>
-
 #include "nest/bird.h"
 #include "nest/route.h"
 #include "nest/protocol.h"
 #include "nest/iface.h"
 #include "nest/cli.h"
 #include "nest/attrs.h"
+#include "lib/alloca.h"
 #include "lib/resource.h"
 #include "lib/string.h"
 
@@ -706,8 +705,8 @@ rta_dump(rta *a)
 {
   static char *rts[] = { "RTS_DUMMY", "RTS_STATIC", "RTS_INHERIT", "RTS_DEVICE",
 			 "RTS_STAT_DEV", "RTS_REDIR", "RTS_RIP",
-			 "RTS_OSPF", "RTS_OSPF_EXT", "RTS_OSPF_IA",
-			 "RTS_OSPF_BOUNDARY", "RTS_BGP" };
+			 "RTS_OSPF", "RTS_OSPF_IA", "RTS_OSPF_EXT1",
+                         "RTS_OSPF_EXT2", "RTS_BGP" };
   static char *rtc[] = { "", " BC", " MC", " AC" };
   static char *rtd[] = { "", " DEV", " HOLE", " UNREACH", " PROHIBIT" };
 
