@@ -39,7 +39,7 @@ struct attr_desc {
 };
 
 static int
-bgp_check_origin(struct bgp_proto *p, byte *a, int len)
+bgp_check_origin(struct bgp_proto *p UNUSED, byte *a UNUSED, int len)
 {
   if (len > 2)
     return 6;
@@ -55,7 +55,7 @@ bgp_format_origin(eattr *a, byte *buf)
 }
 
 static int
-bgp_check_path(struct bgp_proto *p, byte *a, int len)
+bgp_check_path(struct bgp_proto *p UNUSED, byte *a, int len)
 {
   while (len)
     {
@@ -71,7 +71,7 @@ bgp_check_path(struct bgp_proto *p, byte *a, int len)
 }
 
 static int
-bgp_check_next_hop(struct bgp_proto *p, byte *a, int len)
+bgp_check_next_hop(struct bgp_proto *p UNUSED, byte *a, int len)
 {
 #ifdef IPV6
   return -1;
@@ -88,7 +88,7 @@ bgp_check_next_hop(struct bgp_proto *p, byte *a, int len)
 }
 
 static int
-bgp_check_reach_nlri(struct bgp_proto *p, byte *a, int len)
+bgp_check_reach_nlri(struct bgp_proto *p UNUSED, byte *a UNUSED, int len UNUSED)
 {
 #ifdef IPV6
   p->mp_reach_start = a;
@@ -98,7 +98,7 @@ bgp_check_reach_nlri(struct bgp_proto *p, byte *a, int len)
 }
 
 static int
-bgp_check_unreach_nlri(struct bgp_proto *p, byte *a, int len)
+bgp_check_unreach_nlri(struct bgp_proto *p UNUSED, byte *a UNUSED, int len UNUSED)
 {
 #ifdef IPV6
   p->mp_unreach_start = a;
@@ -327,7 +327,7 @@ bgp_new_bucket(struct bgp_proto *p, ea_list *new, unsigned hash)
 {
   struct bgp_bucket *b;
   unsigned ea_size = sizeof(ea_list) + new->count * sizeof(eattr);
-  unsigned ea_size_aligned = ALIGN(ea_size, CPU_STRUCT_ALIGN);
+  unsigned ea_size_aligned = BIRD_ALIGN(ea_size, CPU_STRUCT_ALIGN);
   unsigned size = sizeof(struct bgp_bucket) + ea_size;
   unsigned i;
   byte *dest;
@@ -338,7 +338,7 @@ bgp_new_bucket(struct bgp_proto *p, ea_list *new, unsigned hash)
     {
       eattr *a = &new->attrs[i];
       if (!(a->type & EAF_EMBEDDED))
-	size += ALIGN(sizeof(struct adata) + a->u.ptr->length, CPU_STRUCT_ALIGN);
+	size += BIRD_ALIGN(sizeof(struct adata) + a->u.ptr->length, CPU_STRUCT_ALIGN);
     }
 
   /* Create the bucket and hash it */
@@ -364,7 +364,7 @@ bgp_new_bucket(struct bgp_proto *p, ea_list *new, unsigned hash)
 	  struct adata *na = (struct adata *) dest;
 	  memcpy(na, oa, sizeof(struct adata) + oa->length);
 	  a->u.ptr = na;
-	  dest += ALIGN(sizeof(struct adata) + na->length, CPU_STRUCT_ALIGN);
+	  dest += BIRD_ALIGN(sizeof(struct adata) + na->length, CPU_STRUCT_ALIGN);
 	}
     }
 
@@ -415,7 +415,7 @@ bgp_export_check(struct bgp_proto *p, ea_list *new)
 static struct bgp_bucket *
 bgp_get_bucket(struct bgp_proto *p, ea_list *attrs, int originate)
 {
-  ea_list *t, *new;
+  ea_list *new;
   unsigned i, cnt, hash, code;
   eattr *a, *d;
   u32 seen = 0;
@@ -471,7 +471,7 @@ bgp_get_bucket(struct bgp_proto *p, ea_list *attrs, int originate)
 	    d->u.ptr = z;
 	    break;
 	  }
-	default:
+	default: ;
 	}
       d++;
       new->count++;
@@ -515,7 +515,7 @@ bgp_free_bucket(struct bgp_proto *p, struct bgp_bucket *buck)
 }
 
 void
-bgp_rt_notify(struct proto *P, net *n, rte *new, rte *old, ea_list *attrs)
+bgp_rt_notify(struct proto *P, net *n, rte *new, rte *old UNUSED, ea_list *attrs)
 {
   struct bgp_proto *p = (struct bgp_proto *) P;
   struct bgp_bucket *buck;
@@ -560,7 +560,7 @@ bgp_create_attrs(struct bgp_proto *p, rte *e, ea_list **attrs, struct linpool *p
   ea->count = 4;
 
   bgp_set_attr(ea->attrs, pool, BA_ORIGIN,
-       (rta->source == RTS_OSPF_EXT) ? ORIGIN_INCOMPLETE : ORIGIN_IGP);
+       ((rta->source == RTS_OSPF_EXT1) || (rta->source == RTS_OSPF_EXT2)) ? ORIGIN_INCOMPLETE : ORIGIN_IGP);
 
   if (p->is_internal)
     bgp_set_attr(ea->attrs+1, pool, BA_AS_PATH, 0);
