@@ -63,6 +63,7 @@
 #include "conf/conf.h"
 #include "lib/socket.h"
 #include "lib/resource.h"
+#include "lib/string.h"
 
 #include "bgp.h"
 
@@ -75,8 +76,8 @@ static void bgp_connect(struct bgp_proto *p);
 static void bgp_initiate(struct bgp_proto *p);
 static void bgp_setup_listen_sk(void);
 
-void
-bgp_close(struct bgp_proto *p)
+static void
+bgp_close(struct bgp_proto *p UNUSED)
 {
   ASSERT(bgp_counter);
   bgp_counter--;
@@ -138,7 +139,7 @@ bgp_close_conn(struct bgp_conn *conn)
   conn->keepalive_timer = NULL;
   rfree(conn->hold_timer);
   conn->hold_timer = NULL;
-  sk_close(conn->sk);
+  rfree(conn->sk);
   conn->sk = NULL;
   conn->state = BS_IDLE;
   if (conn->error_flag > 1)
@@ -234,7 +235,7 @@ bgp_sock_err(sock *sk, int err)
     {
     case BS_CONNECT:
     case BS_OPENSENT:
-      sk_close(conn->sk);
+      rfree(conn->sk);
       conn->sk = NULL;
       conn->state = BS_ACTIVE;
       bgp_start_timer(conn->connect_retry_timer, p->cf->connect_retry_time);
@@ -369,10 +370,9 @@ bgp_initiate(struct bgp_proto *p)
  * closes the new connection by sending a Notification message.
  */
 static int
-bgp_incoming_connection(sock *sk, int dummy)
+bgp_incoming_connection(sock *sk, int dummy UNUSED)
 {
   struct proto_config *pc;
-  node *n;
   int match = 0;
 
   DBG("BGP: Incoming connection from %I port %d\n", sk->daddr, sk->dport);
@@ -433,6 +433,19 @@ bgp_start_neighbor(struct bgp_proto *p)
 {
   p->local_addr = p->neigh->iface->addr->ip;
   DBG("BGP: local=%I remote=%I\n", p->local_addr, p->next_hop);
+#ifdef IPV6
+  {
+    struct ifa *a;
+    p->local_link = ipa_or(ipa_build(0xfe80,0,0,0), ipa_and(p->local_addr, ipa_build(0,0,~0,~0)));
+    WALK_LIST(a, p->neigh->iface->addrs)
+      if (a->scope == SCOPE_LINK)
+        {
+	  p->local_link = a->ip;
+	  break;
+	}
+    DBG("BGP: Selected link-level address %I\n", p->local_link);
+  }
+#endif
   bgp_initiate(p);
 }
 
