@@ -41,8 +41,22 @@ bgp_create_open(struct bgp_conn *conn, byte *buf)
   put_u16(buf+1, p->local_as);
   put_u16(buf+3, p->cf->hold_time);
   put_u32(buf+5, p->local_id);
+#ifndef IPV6
   buf[9] = 0;				/* No optional parameters */
   return buf+10;
+#else
+  buf += 9;
+  *buf++ = 8;		/* Optional params len */
+  *buf++ = 2;		/* Option: Capability list */
+  *buf++ = 6;		/* Option length */
+  *buf++ = 1;		/* Capability 1: Multiprotocol extensions */
+  *buf++ = 4;		/* Capability data length */
+  *buf++ = 0;		/* We support AF IPv6 */
+  *buf++ = BGP_AF_IPV6;
+  *buf++ = 0;		/* RFU */
+  *buf++ = 1;		/* and SAFI 1 */
+  return buf;
+#endif
 }
 
 static unsigned int
@@ -179,20 +193,27 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
 	  nh = ea_find(buck->eattrs, EA_CODE(EAP_BGP, BA_NEXT_HOP));
 	  ASSERT(nh);
 	  ip = *(ip_addr *) nh->u.ptr->data;
+	  is_ll = 0;
 	  if (ipa_equal(ip, p->local_addr))
-	    is_ll = 1;
+	    {
+	      is_ll = 1;
+	      ip_ll = p->local_link;
+	    }
 	  else
 	    {
 	      n = neigh_find(&p->p, &ip, 0);
 	      if (n && n->iface == p->neigh->iface)
-		is_ll = 1;
-	      else
-		is_ll = 0;
+		{
+		  /* FIXME: We are assuming the global scope addresses use the lower 64 bits
+		   * as an interface identifier which hasn't necessarily to be true.
+		   */
+		  is_ll = 1;
+	          ip_ll = ipa_or(ipa_build(0xfe800000,0,0,0), ipa_and(ip, ipa_build(0,0,~0,~0)));
+		}
 	    }
 	  if (is_ll)
 	    {
 	      *tmp++ = 32;
-	      ip_ll = ipa_or(ipa_build(0xfe80,0,0,0), ipa_and(ip, ipa_build(0,0,~0,~0)));
 	      ipa_hton(ip);
 	      memcpy(tmp, &ip, 16);
 	      ipa_hton(ip_ll);
@@ -351,8 +372,8 @@ bgp_parse_options(struct bgp_conn *conn, byte *opt, int len)
       switch (opt[0])
 	{
 	case 2:
-	  /* Defined in draft-ietf-idr-bgp4-cap-neg-06 */
-	  /* We can safely ignore all capabilities */
+	  /* Capatibility negotiation as per RFC 2842 */
+	  /* We can safely ignore all capabilities announced */
 	  break;
 	default:
 	  /*
@@ -503,7 +524,6 @@ bgp_do_rx_update(struct bgp_conn *conn,
   rta *a = NULL;
   ip_addr prefix;
   net *n;
-  rte e;
   int err = 0, pxlen;
 
   /* Withdraw routes */
@@ -602,7 +622,7 @@ bgp_do_rx_update(struct bgp_conn *conn,
       if (len < 1 || (*x != 16 && *x != 32) || len < *x + 2)
 	goto bad;
       memcpy(bgp_attach_attr(&a0->eattrs, bgp_linpool, BA_NEXT_HOP, 16), x+1, 16);
-      len -= *x + 1;
+      len -= *x + 2;
       x += *x + 1;
 
       /* Ignore SNPA info */
@@ -766,7 +786,7 @@ bgp_rx_notification(struct bgp_conn *conn, byte *pkt, int len)
 }
 
 static void
-bgp_rx_keepalive(struct bgp_conn *conn, byte *pkt, unsigned len)
+bgp_rx_keepalive(struct bgp_conn *conn)
 {
   struct bgp_proto *p = conn->bgp;
 
@@ -805,7 +825,7 @@ bgp_rx_packet(struct bgp_conn *conn, byte *pkt, unsigned len)
     case PKT_OPEN:		return bgp_rx_open(conn, pkt, len);
     case PKT_UPDATE:		return bgp_rx_update(conn, pkt, len);
     case PKT_NOTIFICATION:      return bgp_rx_notification(conn, pkt, len);
-    case PKT_KEEPALIVE:		return bgp_rx_keepalive(conn, pkt, len);
+    case PKT_KEEPALIVE:		return bgp_rx_keepalive(conn);
     default:			bgp_error(conn, 1, 3, pkt+18, 1);
     }
 }
