@@ -2,6 +2,7 @@
  *	Rest in pieces - RIP protocol
  *
  *	Copyright (c) 1998, 1999 Pavel Machek <pavel@ucw.cz>
+ *	              2004       Ondrej Filip <feela@network.cz>
  *
  *	Can be freely distributed and used under the terms of the GNU GPL.
  *
@@ -11,7 +12,7 @@
 		next hops are only advisory, and they are pretty ugly in IpV6.
 		I suggest just forgetting about them.
 
-	FIXME (nonurgent): fold rip_connection into rip_interface?
+	FIXME: (nonurgent): fold rip_connection into rip_interface?
 
 	FIXME: (nonurgent) allow bigger frequencies than 1 regular update in 6 seconds (?)
 	FIXME: propagation of metric=infinity into main routing table may or may not be good idea.
@@ -62,14 +63,11 @@
 
 #define P ((struct rip_proto *) p)
 #define P_CF ((struct rip_proto_config *)p->cf)
-#define E ((struct rip_entry *) e)
 
 #define TRACE(level, msg, args...) do { if (p->debug & level) { log(L_TRACE "%s: " msg, p->name , ## args); } } while(0)
 
 static struct rip_interface *new_iface(struct proto *p, struct iface *new, unsigned long flags, struct iface_patt *patt);
 
-#define P_NAME p->name
-
 /*
  * Output processing
  *
@@ -79,9 +77,9 @@ static struct rip_interface *new_iface(struct proto *p, struct iface *new, unsig
 static void
 rip_tx_err( sock *s, int err )
 {
-  struct rip_connection *c = s->data;
+  struct rip_connection *c = ((struct rip_interface *)(s->data))->busy;
   struct proto *p = c->proto;
-  log( L_ERR "%s: Unexpected error at rip transmit: %M", P_NAME, err );
+  log( L_ERR "%s: Unexpected error at rip transmit: %M", p->name, err );
 }
 
 /*
@@ -94,7 +92,7 @@ rip_tx_err( sock *s, int err )
  * that could be fixed but it is not real problem).
  */
 static int
-rip_tx_prepare(struct proto *p, ip_addr daddr, struct rip_block *b, struct rip_entry *e, struct rip_interface *rif, int pos )
+rip_tx_prepare(struct proto *p, struct rip_block *b, struct rip_entry *e, struct rip_interface *rif, int pos )
 {
   int metric;
   DBG( "." );
@@ -163,7 +161,7 @@ rip_tx( sock *s )
 
       if (!rif->triggered || (!(e->updated < now-5))) {
 	nullupdate = 0;
-	i = rip_tx_prepare( p, s->daddr, packet->block + i, e, rif, i );
+	i = rip_tx_prepare( p, packet->block + i, e, rif, i );
 	if (i >= maxi) {
 	  FIB_ITERATE_PUT(&c->iter, z);
 	  goto break_loop;
@@ -217,7 +215,7 @@ rip_sendto( struct proto *p, ip_addr daddr, int dport, struct rip_interface *rif
   static int num = 0;
 
   if (rif->busy) {
-    log (L_WARN "%s: Interface %s is much too slow, dropping request", P_NAME, iface->name);
+    log (L_WARN "%s: Interface %s is much too slow, dropping request", p->name, iface->name);
     return;
   }
   c = mb_alloc( p->pool, sizeof( struct rip_connection ));
@@ -267,7 +265,9 @@ rip_rte_update_if_better(rtable *tab, net *net, struct proto *p, rte *new)
   rte *old;
 
   old = rte_find(net, p);
-  if (!old || p->rte_better(new, old))
+  if (!old || p->rte_better(new, old) ||
+      (ipa_equal(old->attrs->from, new->attrs->from) &&
+      (old->u.rip.metric != new->u.rip.metric)) )
     rte_update(tab, net, p, new);
 }
 
@@ -361,21 +361,27 @@ process_block( struct proto *p, struct rip_block *block, ip_addr whotoldme )
   ip_addr network = block->network;
 
   CHK_MAGIC;
-  TRACE(D_ROUTES, "block: %I tells me: %I/??? available, metric %d... ", whotoldme, network, metric );
-  /* FIXME: Why `???'? If prefix is unknown, just don't print it.  [mj] */
+#ifdef IPV6
+  TRACE(D_ROUTES, "block: %I tells me: %I/%d available, metric %d... ",
+      whotoldme, network, block->pxlen, metric );
+#else
+  TRACE(D_ROUTES, "block: %I tells me: %I/%d available, metric %d... ",
+      whotoldme, network, ipa_mklen(block->netmask), metric );
+#endif
+
   if ((!metric) || (metric > P_CF->infinity)) {
 #ifdef IPV6 /* Someone is sedning us nexthop and we are ignoring it */
     if (metric == 0xff)
       { DBG( "IpV6 nexthop ignored" ); return; }
 #endif
-    log( L_WARN "%s: Got metric %d from %I", P_NAME, metric, whotoldme );
+    log( L_WARN "%s: Got metric %d from %I", p->name, metric, whotoldme );
     return;
   }
 
   advertise_entry( p, block, whotoldme );
 }
 
-#define BAD( x ) { log( L_REMOTE "%s: " x, P_NAME ); return 1; }
+#define BAD( x ) { log( L_REMOTE "%s: " x, p->name ); return 1; }
 
 /*
  * rip_process_packet - this is main routine for incoming packets.
@@ -384,7 +390,7 @@ static int
 rip_process_packet( struct proto *p, struct rip_packet *packet, int num, ip_addr whotoldme, int port )
 {
   int i;
-  int native_class = 0, authenticated = 0;
+  int authenticated = 0;
   neighbor *neighbor;
 
   switch( packet->heading.version ) {
@@ -404,12 +410,12 @@ rip_process_packet( struct proto *p, struct rip_packet *packet, int num, ip_addr
           break;
   case RIPCMD_RESPONSE: DBG( "*** Rtable from %I\n", whotoldme ); 
           if (port != P_CF->port) {
-	    log( L_REMOTE "%s: %I send me routing info from port %d", P_NAME, whotoldme, port );
+	    log( L_REMOTE "%s: %I send me routing info from port %d", p->name, whotoldme, port );
 	    return 1;
 	  }
 
 	  if (!(neighbor = neigh_find( p, &whotoldme, 0 )) || neighbor->scope == SCOPE_HOST) {
-	    log( L_REMOTE "%s: %I send me routing info but he is not my neighbor", P_NAME, whotoldme );
+	    log( L_REMOTE "%s: %I send me routing info but he is not my neighbor", p->name, whotoldme );
 	    return 0;
 	  }
 
@@ -457,6 +463,10 @@ rip_rx(sock *s, int size)
   struct proto *p = i->proto;
   int num;
 
+  /* In non-listening mode, just ignore packet */
+  if (i->mode & IM_NOLISTEN)
+    return 1;
+
   CHK_MAGIC;
   DBG( "RIP: message came: %d bytes from %I via %s\n", size, s->faddr, i->iface ? i->iface->name : "(dummy)" );
   size -= sizeof( struct rip_packet_heading );
@@ -568,8 +578,9 @@ rip_start(struct proto *p)
   assert( sizeof(struct rip_block) == 20);
   assert( sizeof(struct rip_block_auth) == 20);
 
-
+#ifdef LOCAL_DEBUG
   P->magic = RIP_MAGIC;
+#endif
   fib_init( &P->rtable, p->pool, sizeof( struct rip_entry ), 0, NULL );
   init_list( &P->connections );
   init_list( &P->garbage );
@@ -602,7 +613,7 @@ static void
 rip_dump(struct proto *p)
 {
   int i;
-  node *w, *e;
+  node *w;
   struct rip_interface *rif;
   i = 0;
 
@@ -614,7 +625,7 @@ rip_dump(struct proto *p)
   i = 0;
   FIB_WALK( &P->rtable, e ) {
     debug( "RIP: entry #%d: ", i++ );
-    rip_dump_entry( E );
+    rip_dump_entry( (struct rip_entry *)e );
   } FIB_WALK_END;
   i = 0;
   WALK_LIST( rif, P->interfaces ) {
@@ -633,14 +644,8 @@ rip_get_route_info(rte *rte, byte *buf, ea_list *attrs)
     bsprintf(buf, " t%04x", tag->u.data);
 }
 
-static int
-rip_want_this_if(struct rip_interface *iface)
-{
-  return 1;
-}
-
 static void
-kill_iface(struct proto *p, struct rip_interface *i)
+kill_iface(struct rip_interface *i)
 {
   DBG( "RIP: Interface %s disappeared\n", i->iface->name);
   rfree(i->sock);
@@ -698,7 +703,7 @@ new_iface(struct proto *p, struct iface *new, unsigned long flags, struct iface_
 
   if (new) {
     if (new->addr->flags & IA_UNNUMBERED)
-      log( L_WARN "%s: rip is not defined over unnumbered links", P_NAME );
+      log( L_WARN "%s: rip is not defined over unnumbered links", p->name );
     if (rif->multicast) {
 #ifndef IPV6
       rif->sock->daddr = ipa_from_u32(0xe0000009);
@@ -715,18 +720,16 @@ new_iface(struct proto *p, struct iface *new, unsigned long flags, struct iface_
 
   if (!ipa_nonzero(rif->sock->daddr)) {
     if (rif->iface)
-      log( L_WARN "%s: interface %s is too strange for me", P_NAME, rif->iface->name );
-  } else
-    if (!(rif->mode & IM_NOLISTEN))
-      if (sk_open(rif->sock)<0) {
-	log( L_ERR "%s: could not listen on %s", P_NAME, rif->iface ? rif->iface->name : "(dummy)" );
-	if (rif->iface) {
-	  rfree(rif->sock);
-	  mb_free(rif);
-	  return NULL;
-	}
-	/* On dummy, we just return non-working socket, so that user gets error every time anyone requests table */
-      }
+      log( L_WARN "%s: interface %s is too strange for me", p->name, rif->iface->name );
+  } else if (sk_open(rif->sock)<0) {
+    log( L_ERR "%s: could not create socket for %s", p->name, rif->iface ? rif->iface->name : "(dummy)" );
+    if (rif->iface) {
+      rfree(rif->sock);
+      mb_free(rif);
+      return NULL;
+    }
+    /* On dummy, we just return non-working socket, so that user gets error every time anyone requests table */
+  }
 
   TRACE(D_EVENTS, "Listening on %s, port %d, mode %s (%I)", rif->iface ? rif->iface->name : "(dummy)", P_CF->port, rif->multicast ? "multicast" : "broadcast", rif->sock->daddr );
   
@@ -764,26 +767,25 @@ rip_if_notify(struct proto *p, unsigned c, struct iface *iface)
     if (i) {
       rem_node(NODE i);
       rfree(i->lock);
-      kill_iface(p, i);
+      kill_iface(i);
     }
   }
   if (c & IF_CHANGE_UP) {
     struct iface_patt *k = iface_patt_match(&P_CF->iface_list, iface);
     struct object_lock *lock;
+    struct rip_patt *PATT = (struct rip_patt *) k;
 
     if (!k) return; /* We are not interested in this interface */
 
     lock = olock_new( p->pool );
+    if (!(PATT->mode & IM_BROADCAST) && (iface->flags & IF_MULTICAST))
 #ifndef IPV6
-    lock->addr = ipa_from_u32(0xe0000009);	/* This is okay: we
-						   may actually use
-						   other address, but
-						   we do not want two
-						   rips at one time,
-						   anyway. */
+      lock->addr = ipa_from_u32(0xe0000009);
 #else
-    ip_pton("FF02::9", &lock->addr);
+      ip_pton("FF02::9", &lock->addr);
 #endif
+    else
+      lock->addr = iface->addr->brd;
     lock->port = P_CF->port;
     lock->iface = iface;
     lock->hook = rip_real_if_add;
@@ -794,7 +796,7 @@ rip_if_notify(struct proto *p, unsigned c, struct iface *iface)
 }
 
 static struct ea_list *
-rip_gen_attrs(struct proto *p, struct linpool *pool, int metric, u16 tag)
+rip_gen_attrs(struct linpool *pool, int metric, u16 tag)
 {
   struct ea_list *l = lp_alloc(pool, sizeof(struct ea_list) + 2*sizeof(eattr));
 
@@ -819,7 +821,7 @@ rip_import_control(struct proto *p, struct rte **rt, struct ea_list **attrs, str
     return 1;
 
   if ((*rt)->attrs->source != RTS_RIP) {
-    struct ea_list *new = rip_gen_attrs(p, pool, 1, 0);
+    struct ea_list *new = rip_gen_attrs(pool, 1, 0);
     new->next = *attrs;
     *attrs = new;
   }
@@ -829,8 +831,7 @@ rip_import_control(struct proto *p, struct rte **rt, struct ea_list **attrs, str
 static struct ea_list *
 rip_make_tmp_attrs(struct rte *rt, struct linpool *pool)
 {
-  struct proto *p = rt->attrs->proto;
-  return rip_gen_attrs(p, pool, rt->u.rip.metric, rt->u.rip.tag);
+  return rip_gen_attrs(pool, rt->u.rip.metric, rt->u.rip.tag);
 }
 
 static void 
@@ -852,7 +853,7 @@ rip_rt_notify(struct proto *p, struct network *net, struct rte *new, struct rte
   if (old) {
     struct rip_entry *e = fib_find( &P->rtable, &net->n.prefix, net->n.pxlen );
     if (!e)
-      log( L_BUG "%s: Deleting nonexistent entry?!", P_NAME );
+      log( L_BUG "%s: Deleting nonexistent entry?!", p->name );
     fib_delete( &P->rtable, e );
   }
 
@@ -861,7 +862,7 @@ rip_rt_notify(struct proto *p, struct network *net, struct rte *new, struct rte
 #if 0
     /* This can happen since feeding of protocols is asynchronous */
     if (fib_find( &P->rtable, &net->n.prefix, net->n.pxlen ))
-      log( L_BUG "%s: Inserting entry which is already there?", P_NAME );
+      log( L_BUG "%s: Inserting entry which is already there?", p->name );
 #endif
     e = fib_get( &P->rtable, &net->n.prefix, net->n.pxlen );
 
@@ -886,6 +887,14 @@ rip_rt_notify(struct proto *p, struct network *net, struct rte *new, struct rte
   }
 }
 
+static int
+rip_rte_same(struct rte *new, struct rte *old)
+{
+  /* new->attrs == old->attrs always */
+  return new->u.rip.metric == old->u.rip.metric;
+}
+
+
 static int
 rip_rte_better(struct rte *new, struct rte *old)
 {
@@ -914,7 +923,7 @@ rip_rte_better(struct rte *new, struct rte *old)
  * walks the list.
  */
 static void
-rip_rte_insert(net *net, rte *rte)
+rip_rte_insert(net *net UNUSED, rte *rte)
 {
   struct proto *p = rte->attrs->proto;
   CHK_MAGIC;
@@ -926,7 +935,7 @@ rip_rte_insert(net *net, rte *rte)
  * rip_rte_remove - link list maintenance
  */
 static void
-rip_rte_remove(net *net, rte *rte)
+rip_rte_remove(net *net UNUSED, rte *rte)
 {
   struct proto *p = rte->attrs->proto;
   CHK_MAGIC;
@@ -943,6 +952,7 @@ rip_init_instance(struct proto *p)
   p->make_tmp_attrs = rip_make_tmp_attrs;
   p->store_tmp_attrs = rip_store_tmp_attrs;
   p->rte_better = rip_rte_better;
+  p->rte_same = rip_rte_same;
   p->rte_insert = rip_rte_insert;
   p->rte_remove = rip_rte_remove;
 }
@@ -986,9 +996,6 @@ rip_reconfigure(struct proto *p, struct proto_config *c)
 
   if (!iface_patts_equal(&P_CF->iface_list, &new->iface_list, (void *) rip_pat_compare))
     return 0;
-  if (!password_same(P_CF->passwords, 
-		     new->passwords))
-    return 0;
   return !memcmp(((byte *) P_CF) + generic,
                  ((byte *) new) + generic,
                  sizeof(struct rip_proto_config) - generic);
