@@ -114,13 +114,13 @@ nl_get_reply(void)
       if (NLMSG_OK(nl_last_hdr, nl_last_size))
 	{
 	  struct nlmsghdr *h = nl_last_hdr;
+	  nl_last_hdr = NLMSG_NEXT(h, nl_last_size);
 	  if (h->nlmsg_seq != nl_sync_seq)
 	    {
 	      log(L_WARN "nl_get_reply: Ignoring out of sequence netlink packet (%x != %x)",
 		  h->nlmsg_seq, nl_sync_seq);
 	      continue;
 	    }
-	  nl_last_hdr = NLMSG_NEXT(h, nl_last_size);
 	  return h;
 	}
       if (nl_last_size)
@@ -340,11 +340,6 @@ nl_parse_addr(struct nlmsghdr *h)
       return;
     }
 
-#ifdef IPV6
-  if (i->ifa_scope == RT_SCOPE_LINK)
-    return;
-#endif
-
   ifi = if_find_by_index(i->ifa_index);
   if (!ifi)
     {
@@ -414,7 +409,7 @@ nl_parse_addr(struct nlmsghdr *h)
 }
 
 void
-krt_if_scan(struct kif_proto *p)
+krt_if_scan(struct kif_proto *p UNUSED)
 {
   struct nlmsghdr *h;
 
@@ -523,7 +518,7 @@ nl_send_route(struct krt_proto *p, rte *e, int new)
 }
 
 void
-krt_set_notify(struct krt_proto *p, net *n, rte *new, rte *old)
+krt_set_notify(struct krt_proto *p, net *n UNUSED, rte *new, rte *old)
 {
   if (old && new)
     {
@@ -546,7 +541,7 @@ krt_set_notify(struct krt_proto *p, net *n, rte *new, rte *old)
     }
 }
 
-struct iface *
+static struct iface *
 krt_temp_iface(struct krt_proto *p, unsigned index)
 {
   struct iface *i, *j;
@@ -630,7 +625,7 @@ nl_parse_route(struct nlmsghdr *h, int scan)
   else
     oif = ~0;
 
-  DBG("Got %I/%d, type=%d, oif=%d, table=%d, proto=%s\n", dst, i->rtm_dst_len, i->rtm_type, oif, i->rtm_table, p->p.name);
+  DBG("Got %I/%d, type=%d, oif=%d, table=%d, prid=%d, proto=%s\n", dst, i->rtm_dst_len, i->rtm_type, oif, i->rtm_table, i->rtm_protocol, p->p.name);
 
   switch (i->rtm_protocol)
     {
@@ -641,6 +636,10 @@ nl_parse_route(struct nlmsghdr *h, int scan)
       DBG("Route originated in kernel, ignoring\n");
       return;
     case RTPROT_BIRD:
+#ifdef IPV6
+    case RTPROT_BOOT:
+      /* Current Linux kernels don't remember rtm_protocol for IPv6 routes and supply RTPROT_BOOT instead */
+#endif
       if (!scan)
 	{
 	  DBG("Echo of our own route, ignoring\n");
@@ -727,7 +726,7 @@ nl_parse_route(struct nlmsghdr *h, int scan)
 }
 
 void
-krt_scan_fire(struct krt_proto *p)	/* CONFIG_ALL_TABLES_AT_ONCE => p is NULL */
+krt_scan_fire(struct krt_proto *p UNUSED)	/* CONFIG_ALL_TABLES_AT_ONCE => p is NULL */
 {
   struct nlmsghdr *h;
 
@@ -772,7 +771,7 @@ nl_async_msg(struct nlmsghdr *h)
 }
 
 static int
-nl_async_hook(sock *sk, int size)
+nl_async_hook(sock *sk, int size UNUSED)
 {
   struct iovec iov = { nl_async_rx_buffer, NL_RX_SIZE };
   struct sockaddr_nl sa;
@@ -872,7 +871,7 @@ nl_open_async(void)
 static u8 nl_cf_table[(NL_NUM_TABLES+7) / 8];
 
 void
-krt_scan_preconfig(struct config *c)
+krt_scan_preconfig(struct config *c UNUSED)
 {
   bzero(&nl_cf_table, sizeof(nl_cf_table));
 }
@@ -892,6 +891,8 @@ krt_scan_construct(struct krt_config *x)
 {
 #ifndef IPV6
   x->scan.table_id = RT_TABLE_MAIN;
+#else
+  x->scan.table_id = 254;
 #endif
 }
 
@@ -908,12 +909,12 @@ krt_scan_start(struct krt_proto *p, int first)
 }
 
 void
-krt_scan_shutdown(struct krt_proto *p, int last)
+krt_scan_shutdown(struct krt_proto *p UNUSED, int last UNUSED)
 {
 }
 
 void
-krt_if_start(struct kif_proto *p)
+krt_if_start(struct kif_proto *p UNUSED)
 {
   nl_open();
   nl_open_async();
