@@ -1,13 +1,15 @@
 /*
  *	BIRD Internet Routing Daemon -- Unix I/O
  *
- *	(c) 1998--2000 Martin Mares <mj@ucw.cz>
+ *	(c) 1998--2004 Martin Mares <mj@ucw.cz>
+ *      (c) 2004       Ondrej Filip <feela@network.cz>
  *
  *	Can be freely distributed and used under the terms of the GNU GPL.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -297,6 +299,30 @@ tm_shot(void)
     }
 }
 
+/**
+ * tm_parse_datetime - parse a date and time
+ * @x: datetime string
+ *
+ * tm_parse_datetime() takes a textual representation of
+ * a date and time (dd-mm-yyyy hh:mm:ss)
+ * and converts it to the corresponding value of type &bird_clock_t.
+ */
+bird_clock_t
+tm_parse_datetime(char *x)
+{
+  struct tm tm;
+  int n;
+  time_t t;
+
+  if (sscanf(x, "%d-%d-%d %d:%d:%d%n", &tm.tm_mday, &tm.tm_mon, &tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec, &n) != 6 || x[n])
+    return tm_parse_date(x);
+  tm.tm_mon--;
+  tm.tm_year -= 1900;
+  t = mktime(&tm);
+  if (t == (time_t) -1)
+    return 0;
+  return t;
+}
 /**
  * tm_parse_date - parse a date
  * @x: date string
@@ -391,27 +417,83 @@ tm_format_reltime(char *x, bird_clock_t t)
  * (@rx_hook), when the contents of the transmit buffer have been transmitted
  * (@tx_hook) and when an error or connection close occurs (@err_hook).
  *
- * You should not use rfree() from inside a socket hook, please use sk_close() instead.
+ * Freeing of sockets from inside socket hooks is perfectly safe.
  */
 
 #ifndef SOL_IP
 #define SOL_IP IPPROTO_IP
 #endif
 
+#ifndef SOL_IPV6
+#define SOL_IPV6 IPPROTO_IPV6
+#endif
+
+#ifndef IPV6_ADD_MEMBERSHIP
+#define IPV6_ADD_MEMBERSHIP IP_ADD_MEMBERSHIP
+#endif
+
 static list sock_list;
+static struct birdsock *current_sock;
+static int sock_recalc_fdsets_p;
+
+static inline sock *
+sk_next(sock *s)
+{
+  if (!s->n.next->next)
+    return NULL;
+  else
+    return SKIP_BACK(sock, n, s->n.next);
+}
+
+static void
+sk_alloc_bufs(sock *s)
+{
+  if (!s->rbuf && s->rbsize)
+    s->rbuf = s->rbuf_alloc = xmalloc(s->rbsize);
+  s->rpos = s->rbuf;
+  if (!s->tbuf && s->tbsize)
+    s->tbuf = s->tbuf_alloc = xmalloc(s->tbsize);
+  s->tpos = s->ttx = s->tbuf;
+}
+
+static void
+sk_free_bufs(sock *s)
+{
+  if (s->rbuf_alloc)
+    {
+      xfree(s->rbuf_alloc);
+      s->rbuf = s->rbuf_alloc = NULL;
+    }
+  if (s->tbuf_alloc)
+    {
+      xfree(s->tbuf_alloc);
+      s->tbuf = s->tbuf_alloc = NULL;
+    }
+}
 
 static void
 sk_free(resource *r)
 {
   sock *s = (sock *) r;
 
+  sk_free_bufs(s);
   if (s->fd >= 0)
     {
       close(s->fd);
+      if (s == current_sock)
+	current_sock = sk_next(s);
       rem_node(&s->n);
+      sock_recalc_fdsets_p = 1;
     }
 }
 
+void
+sk_reallocate(sock *s)
+{
+  sk_free_bufs(s);
+  sk_alloc_bufs(s);
+}
+
 static void
 sk_dump(resource *r)
 {
@@ -463,29 +545,37 @@ sk_new(pool *p)
   s->tbsize = 0;
   s->err_hook = NULL;
   s->fd = -1;
-  s->entered = 0;
+  s->rbuf_alloc = s->tbuf_alloc = NULL;
   return s;
 }
 
-#define ERR(x) do { err = x; goto bad; } while(0)
-#define WARN(x) log(L_WARN "sk_setup: %s: %m", x)
+static void
+sk_insert(sock *s)
+{
+  add_tail(&sock_list, &s->n);
+  sock_recalc_fdsets_p = 1;
+}
 
 #ifdef IPV6
 
 void
 fill_in_sockaddr(sockaddr *sa, ip_addr a, unsigned port)
 {
+  memset (sa, 0, sizeof (struct sockaddr_in6));
   sa->sin6_family = AF_INET6;
   sa->sin6_port = htons(port);
   sa->sin6_flowinfo = 0;
+#ifdef HAVE_SIN_LEN
+  sa->sin6_len = sizeof(struct sockaddr_in6);
+#endif
   set_inaddr(&sa->sin6_addr, a);
 }
 
 void
-get_sockaddr(sockaddr *sa, ip_addr *a, unsigned *port)
+get_sockaddr(struct sockaddr_in6 *sa, ip_addr *a, unsigned *port, int check)
 {
-  if (sa->sin6_family != AF_INET6)
-    bug("get_sockaddr called for wrong address family");
+  if (check && sa->sin6_family != AF_INET6)
+    bug("get_sockaddr called for wrong address family (%d)", sa->sin6_family);
   if (port)
     *port = ntohs(sa->sin6_port);
   memcpy(a, &sa->sin6_addr, sizeof(*a));
@@ -497,16 +587,20 @@ get_sockaddr(sockaddr *sa, ip_addr *a, unsigned *port)
 void
 fill_in_sockaddr(sockaddr *sa, ip_addr a, unsigned port)
 {
+  memset (sa, 0, sizeof (struct sockaddr_in));
   sa->sin_family = AF_INET;
   sa->sin_port = htons(port);
+#ifdef HAVE_SIN_LEN
+  sa->sin_len = sizeof(struct sockaddr_in);
+#endif
   set_inaddr(&sa->sin_addr, a);
 }
 
 void
-get_sockaddr(sockaddr *sa, ip_addr *a, unsigned *port)
+get_sockaddr(struct sockaddr_in *sa, ip_addr *a, unsigned *port, int check)
 {
-  if (sa->sin_family != AF_INET)
-    bug("get_sockaddr called for wrong address family");
+  if (check && sa->sin_family != AF_INET)
+    bug("get_sockaddr called for wrong address family (%d)", sa->sin_family);
   if (port)
     *port = ntohs(sa->sin_port);
   memcpy(a, &sa->sin_addr.s_addr, sizeof(*a));
@@ -515,6 +609,9 @@ get_sockaddr(sockaddr *sa, ip_addr *a, unsigned *port)
 
 #endif
 
+#define ERR(x) do { err = x; goto bad; } while(0)
+#define WARN(x) log(L_WARN "sk_setup: %s: %m", x)
+
 static char *
 sk_setup(sock *s)
 {
@@ -535,25 +632,16 @@ sk_setup(sock *s)
     WARN("IP_TOS");
   if (s->ttl >= 0 && setsockopt(fd, SOL_IP, IP_TTL, &s->ttl, sizeof(s->ttl)) < 0)
     ERR("IP_TTL");
+#ifdef CONFIG_UNIX_DONTROUTE
   if (s->ttl == 1 && setsockopt(fd, SOL_SOCKET, SO_DONTROUTE, &one, sizeof(one)) < 0)
     ERR("SO_DONTROUTE");
+#endif 
 #endif
   err = NULL;
 bad:
   return err;
 }
 
-static void
-sk_alloc_bufs(sock *s)
-{
-  if (!s->rbuf && s->rbsize)
-    s->rbuf = mb_alloc(s->pool, s->rbsize);
-  s->rpos = s->rbuf;
-  if (!s->tbuf && s->tbsize)
-    s->tbuf = mb_alloc(s->pool, s->tbsize);
-  s->tpos = s->ttx = s->tbuf;
-}
-
 static void
 sk_tcp_connected(sock *s)
 {
@@ -577,8 +665,8 @@ sk_passive_connected(sock *s, struct sockaddr *sa, int al, int type)
       t->rbsize = s->rbsize;
       t->tbsize = s->tbsize;
       if (type == SK_TCP)
-	get_sockaddr((sockaddr *) sa, &t->daddr, &t->dport);
-      add_tail(&sock_list, &t->n);
+	get_sockaddr((sockaddr *) sa, &t->daddr, &t->dport, 1);
+      sk_insert(t);
       if (err = sk_setup(t))
 	{
 	  log(L_ERR "Incoming connection: %s: %m", err);
@@ -645,6 +733,7 @@ sk_open(sock *s)
 
   if (err = sk_setup(s))
     goto bad;
+
   switch (type)
     {
     case SK_UDP:
@@ -682,7 +771,7 @@ sk_open(sock *s)
 	    mreq.ipv6mr_ifindex = s->iface->index;
 #else
 	    mreq.ipv6mr_interface = s->iface->index;
-#endif
+#endif /* CONFIG_IPV6_GLIBC_20 */
 	    if (setsockopt(fd, SOL_IPV6, IPV6_ADD_MEMBERSHIP, &mreq, sizeof(mreq)) < 0)
 	      ERR("IPV6_ADD_MEMBERSHIP");
 	  }
@@ -691,7 +780,7 @@ sk_open(sock *s)
 	ASSERT(s->iface && s->iface->addr);
 	if (err = sysio_mcast_join(s))
 	  goto bad;
-#endif
+#endif /* IPV6 */
       break;
       }
     }
@@ -708,7 +797,11 @@ sk_open(sock *s)
 	    ERR("SO_REUSEADDR");
 	}
       fill_in_sockaddr(&sa, s->saddr, port);
+#ifdef CONFIG_SKIP_MC_BIND
+      if (type == SK_IP && bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0)
+#else
       if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0)
+#endif
 	ERR("bind");
     }
   fill_in_sockaddr(&sa, s->daddr, s->dport);
@@ -748,7 +841,7 @@ sk_open(sock *s)
 #endif
     }
 
-  add_tail(&sock_list, &s->n);
+  sk_insert(s);
   return 0;
 
 bad:
@@ -778,7 +871,7 @@ sk_open_unix(sock *s, char *name)
     ERR("bind");
   if (listen(fd, 8))
     ERR("listen");
-  add_tail(&sock_list, &s->n);
+  sk_insert(s);
   return 0;
 
 bad:
@@ -788,23 +881,6 @@ bad:
   return -1;
 }
 
-/**
- * sk_close - close a socket
- * @s: a socket
- *
- * If sk_close() has been called from outside of any socket hook,
- * it translates to a rfree(), else it just marks the socket for
- * deletion as soon as the socket hook returns.
- */
-void
-sk_close(sock *s)
-{
-  if (s && s->entered)
-    s->type = SK_DELETED;
-  else
-    rfree(s);
-}
-
 static int
 sk_maybe_write(sock *s)
 {
@@ -822,6 +898,7 @@ sk_maybe_write(sock *s)
 	    {
 	      if (errno != EINTR && errno != EAGAIN)
 		{
+                  s->ttx = s->tpos;	/* empty tx buffer */
 		  s->err_hook(s, errno);
 		  return -1;
 		}
@@ -841,11 +918,13 @@ sk_maybe_write(sock *s)
 	if (s->tbuf == s->tpos)
 	  return 1;
 	fill_in_sockaddr(&sa, s->faddr, s->fport);
+
 	e = sendto(s->fd, s->tbuf, s->tpos - s->tbuf, 0, (struct sockaddr *) &sa, sizeof(sa));
 	if (e < 0)
 	  {
 	    if (errno != EINTR && errno != EAGAIN)
 	      {
+                s->ttx = s->tpos;	/* empty tx buffer */
 		s->err_hook(s, errno);
 		return -1;
 	      }
@@ -932,15 +1011,17 @@ sk_read(sock *s)
 	  {
 	    s->rpos += c;
 	    if (s->rx_hook(s, s->rpos - s->rbuf))
-	      s->rpos = s->rbuf;
+	      {
+		/* We need to be careful since the socket could have been deleted by the hook */
+		if (current_sock == s)
+		  s->rpos = s->rbuf;
+	      }
 	    return 1;
 	  }
 	return 0;
       }
     case SK_MAGIC:
       return s->rx_hook(s, 0);
-    case SK_DELETED:
-      return 0;
     default:
       {
 	sockaddr sa;
@@ -954,14 +1035,14 @@ sk_read(sock *s)
 	    return 0;
 	  }
 	s->rpos = s->rbuf + e;
-	get_sockaddr(&sa, &s->faddr, &s->fport);
+	get_sockaddr(&sa, &s->faddr, &s->fport, 1);
 	s->rx_hook(s, e);
 	return 1;
       }
     }
 }
 
-static void
+static int
 sk_write(sock *s)
 {
   switch (s->type)
@@ -970,17 +1051,19 @@ sk_write(sock *s)
       {
 	sockaddr sa;
 	fill_in_sockaddr(&sa, s->daddr, s->dport);
-	if (connect(s->fd, (struct sockaddr *) &sa, sizeof(sa)) >= 0)
+	if (connect(s->fd, (struct sockaddr *) &sa, sizeof(sa)) >= 0 || errno == EISCONN)
 	  sk_tcp_connected(s);
 	else if (errno != EINTR && errno != EAGAIN && errno != EINPROGRESS)
 	  s->err_hook(s, errno);
-	break;
+	return 0;
       }
-    case SK_DELETED:
-      return;
     default:
-      while (s->ttx != s->tpos && sk_maybe_write(s) > 0)
-	s->tx_hook(s);
+      if (s->ttx != s->tpos && sk_maybe_write(s) > 0)
+	{
+	  s->tx_hook(s);
+	  return 1;
+	}
+      return 0;
     }
 }
 
@@ -1030,10 +1113,9 @@ io_loop(void)
   time_t tout;
   int hi, events;
   sock *s;
-  node *n, *p;
+  node *n;
 
-  FD_ZERO(&rd);
-  FD_ZERO(&wr);
+  sock_recalc_fdsets_p = 1;
   for(;;)
     {
       events = ev_run_list(&global_event_list);
@@ -1047,6 +1129,13 @@ io_loop(void)
       timo.tv_sec = events ? 0 : tout - now;
       timo.tv_usec = 0;
 
+      if (sock_recalc_fdsets_p)
+	{
+	  sock_recalc_fdsets_p = 0;
+	  FD_ZERO(&rd);
+	  FD_ZERO(&wr);
+	}
+
       hi = 0;
       WALK_LIST(n, sock_list)
 	{
@@ -1057,12 +1146,16 @@ io_loop(void)
 	      if (s->fd > hi)
 		hi = s->fd;
 	    }
+	  else
+	    FD_CLR(s->fd, &rd);
 	  if (s->tx_hook && s->ttx != s->tpos)
 	    {
 	      FD_SET(s->fd, &wr);
 	      if (s->fd > hi)
 		hi = s->fd;
 	    }
+	  else
+	    FD_CLR(s->fd, &wr);
 	}
 
       /*
@@ -1100,25 +1193,50 @@ io_loop(void)
 	}
       if (hi)
 	{
-	  WALK_LIST_DELSAFE(n, p, sock_list)
+	  current_sock = SKIP_BACK(sock, n, HEAD(sock_list));	/* guaranteed to be non-empty */
+	  while (current_sock)
 	    {
-	      s = SKIP_BACK(sock, n, n);
-	      s->entered = 1;
+	      sock *s = current_sock;
+	      int e;
 	      if (FD_ISSET(s->fd, &rd))
-		{
-		  FD_CLR(s->fd, &rd);
-		  while (sk_read(s))
-		    ;
-		}
-	      if (s->type != SK_DELETED && FD_ISSET(s->fd, &wr))
-		{
-		  FD_CLR(s->fd, &wr);
-		  sk_write(s);
-		}
-	      s->entered = 0;
-	      if (s->type == SK_DELETED)
-		rfree(s);
+		do
+		  {
+		    e = sk_read(s);
+		    if (s != current_sock)
+		      goto next;
+		  }
+		while (e);
+	      if (FD_ISSET(s->fd, &wr))
+		do
+		  {
+		    e = sk_write(s);
+		    if (s != current_sock)
+		      goto next;
+		  }
+		while (e);
+	      current_sock = sk_next(s);
+	    next: ;
 	    }
 	}
     }
 }
+
+void
+test_old_bird(char *path)
+{
+  int fd;
+  struct sockaddr_un sa;
+
+  fd = socket(AF_UNIX, SOCK_STREAM, 0);
+
+  if (fd < 0)
+    die("Cannot create socket: %m");
+  bzero(&sa, sizeof(sa));
+  sa.sun_family = AF_UNIX;
+  strcpy(sa.sun_path, path);
+  if (connect(fd, (struct sockaddr *) &sa, SUN_LEN(&sa)) == 0)
+    die("I found another BIRD running.");
+  close(fd);
+}
+
+
