@@ -48,6 +48,7 @@
 #include "nest/protocol.h"
 #include "lib/timer.h"
 #include "conf/conf.h"
+#include "lib/string.h"
 
 #include "unix.h"
 #include "krt.h"
@@ -78,7 +79,7 @@ static timer *kif_scan_timer;
 static bird_clock_t kif_last_shot;
 
 static void
-kif_preconfig(struct protocol *P, struct config *c)
+kif_preconfig(struct protocol *P UNUSED, struct config *c UNUSED)
 {
   cf_kif = NULL;
 }
@@ -494,8 +495,10 @@ krt_got_route(struct krt_proto *p, rte *e)
 {
   rte *old;
   net *net = e->net;
-  int src = e->u.krt.src;
   int verdict;
+#ifdef KRT_ALLOW_LEARN
+  int src = e->u.krt.src;
+#endif
 
 #ifdef CONFIG_AUTO_ROUTES
   if (e->attrs->dest == RTD_DEVICE)
@@ -512,7 +515,10 @@ krt_got_route(struct krt_proto *p, rte *e)
       if (KRT_CF->learn)
 	krt_learn_scan(p, e);
       else
-	krt_trace_in(p, e, "alien route, ignored");
+	{
+	  krt_trace_in(p, e, "alien route, ignored");
+	  rte_free(e);
+	}
       return;
     }
 #endif
@@ -521,6 +527,7 @@ krt_got_route(struct krt_proto *p, rte *e)
     {
       /* Route to this destination was already seen. Strange, but it happens... */
       krt_trace_in(p, e, "already seen");
+      rte_free(e);
       return;
     }
 
@@ -611,7 +618,7 @@ krt_prune(struct krt_proto *p)
 }
 
 void
-krt_got_route_async(struct krt_proto *p, rte *e, int new)
+krt_got_route_async(struct krt_proto *p, rte *e, int new UNUSED)
 {
   net *net = e->net;
   int src = e->u.krt.src;
@@ -641,7 +648,7 @@ krt_got_route_async(struct krt_proto *p, rte *e, int new)
  */
 
 static void
-krt_scan(timer *t)
+krt_scan(timer *t UNUSED)
 {
   struct krt_proto *p;
 
@@ -673,7 +680,7 @@ krt_scan(timer *t)
  */
 
 static void
-krt_notify(struct proto *P, net *net, rte *new, rte *old, struct ea_list *attrs)
+krt_notify(struct proto *P, net *net, rte *new, rte *old, struct ea_list *attrs UNUSED)
 {
   struct krt_proto *p = (struct krt_proto *) P;
 
@@ -698,7 +705,7 @@ krt_notify(struct proto *P, net *net, rte *new, rte *old, struct ea_list *attrs)
 struct proto_config *cf_krt;
 
 static void
-krt_preconfig(struct protocol *P, struct config *c)
+krt_preconfig(struct protocol *P UNUSED, struct config *c)
 {
   cf_krt = NULL;
   krt_scan_preconfig(c);
